# load packages in memory
library(tidyverse)
library(here)
library(MuMIn)



# load data ---------------------------------------------------------------


bike <- read_csv(here("data/bike_raw.csv")) %>% 
  as.data.frame()




# make predictions --------------------------------------------------------

# set up dataframe
bike_predicted <- data.frame(
    Strecke = sort(rep(unique(bike$Strecke), 8)),
    Monat = rep(c(3:10), length(unique(bike$Strecke))),
    Jahr = rep(c(
      2020, 2020, 2020, 2020, 2019, 2019, 2019, 2019
    ),
    length(unique(bike$Strecke))),
    recorded = NA,
    predicted = NA,
    predicted_with_sun = NA,
    predicted_with_sun_interaction = NA,
    dAICc_mod_0 = NA,
    dAICc_mod_Jahr = NA,
    dAICc_mod_Sonne = NA,
    dAICc_mod_Interaktion = NA
  )

# fill in type
bike_predicted$type <- NA
for (i in unique(bike_predicted$Strecke)) {
  bike_predicted$type[bike_predicted$Strecke == i] <-
    bike[bike$Strecke == i, "type"][1]
}

# loop through rows and predict
for(i in 1:nrow(bike_predicted)) {
  if (sum(
    bike$Strecke == bike_predicted[i, "Strecke"] &
    bike$Jahr == bike_predicted[i, "Jahr"] &
    bike$Monat == bike_predicted[i, "Monat"]
  ) > 0) {
    bike_predicted[i, "recorded"] <-
      bike[bike$Strecke == bike_predicted[i, "Strecke"] &
             bike$Jahr == bike_predicted[i, "Jahr"] &
             bike$Monat == bike_predicted[i, "Monat"], "N"] 

    if (sum(
      bike$Strecke == bike_predicted[i, "Strecke"] &
      bike$Monat == bike_predicted[i, "Monat"] &
      bike$Jahr != bike_predicted[i, "Jahr"]
    ) > 3) {
      bike_predicted[i, "predicted"] <- 
        glm(
          N ~ Jahr,
          data = subset(
            bike,
            bike$Strecke == bike_predicted[i, "Strecke"] &
              bike$Monat == bike_predicted[i, "Monat"] &
              bike$Jahr != bike_predicted[i, "Jahr"]),
          family = "poisson") %>%
        predict(newdata = data.frame(Jahr = bike_predicted[i, "Jahr"]),
                type = "response") 
      
      if (!sum(is.na(
        subset(
          bike,
          bike$Strecke == bike_predicted[i, "Strecke"] &
          bike$Monat == bike_predicted[i, "Monat"]
        )[, "Sonnenscheindauer"]
      )) > 0) {
        bike_predicted[i, "predicted_with_sun"] <-
          glm(
            N ~ Jahr + Sonnenscheindauer,
            data = subset(
              bike,
              Strecke == bike_predicted[i, "Strecke"] &
                Monat == bike_predicted[i, "Monat"] &
                Jahr != bike_predicted[i, "Jahr"]),
            family = "poisson") %>%
          predict(
            newdata = data.frame(Jahr = bike_predicted[i, "Jahr"],
                                 Sonnenscheindauer =
                                   bike[bike$Strecke == bike_predicted[i, "Strecke"] &
                                          bike$Jahr == bike_predicted[i, "Jahr"] &
                                          bike$Monat == bike_predicted[i, "Monat"],
                                        "Sonnenscheindauer"]),
            type = "response")
        
        bike_predicted[i, "predicted_with_sun_interaction"] <-
          glm(
            N ~ Jahr * Sonnenscheindauer,
            data = subset(
              bike,
              Strecke == bike_predicted[i, "Strecke"]
              &
                Monat == bike_predicted[i, "Monat"] &
                Jahr != bike_predicted[i, "Jahr"]
            ),
            family = "poisson"
          ) %>%
          predict(
            newdata = data.frame(Jahr = bike_predicted[i, "Jahr"],
                                 Sonnenscheindauer =
                                   bike[bike$Strecke == bike_predicted[i, "Strecke"] &
                                          bike$Jahr == bike_predicted[i, "Jahr"] &
                                          bike$Monat == bike_predicted[i, "Monat"],
                                        "Sonnenscheindauer"]),
            type = "response"
          ) 
        
        testdat <-
          subset(
            bike,
            Strecke == bike_predicted[i, "Strecke"] &
              Monat == bike_predicted[i, "Monat"] &
              Jahr != bike_predicted[i, "Jahr"]
          )[, c("N", "Jahr", "Sonnenscheindauer")]
        
        mod0 <- glm(N ~ 1, data = testdat, family = "poisson")
        mod_Jahr <- glm(N ~ Jahr, data = testdat, family = "poisson")
        mod_Sonne <-
          glm(N ~ Jahr + Sonnenscheindauer,
              data = testdat,
              family = "poisson")
        mod_Interaktion <-
          glm(N ~ Jahr * Sonnenscheindauer,
              data = testdat,
              family = "poisson")
        tapl <- AICc(mod0, 
                     mod_Jahr, mod_Sonne, mod_Interaktion)
        bike_predicted[i, c(
         "dAICc_mod_0",
          "dAICc_mod_Jahr",
          "dAICc_mod_Sonne",
          "dAICc_mod_Interaktion"
        )] <- c(tapl[, 2] - min(tapl[, 2]))
      }
    }
  }
}

# compare models
test <- drop_na(bike_predicted) %>% 
  select(dAICc_mod_0, dAICc_mod_Jahr, dAICc_mod_Sonne , dAICc_mod_Interaktion)
  

nr_best <- table(apply(test, 1, which.min)) 
names(nr_best) <- c("0", "1", "2", "3")

# which model performs best
paste("Model", names(which.max(nr_best)), "generally performs best")

# in how many cases, without null model
paste("Model", names(which.max(nr_best)), "performs best in", 
      round((max(nr_best)/sum(nr_best))*100, 1), "% of cases" )



# save data ---------------------------------------------------------------


write_csv(bike_predicted, here("data/bike_predicted.csv"))
